# This Python file uses the following encoding: utf-8
#!/usr/bin/env python3.10
# -*- coding: utf-8 -*-

from config import api_id, api_hash, bot_token
from pyrogram import Client, compose, idle, filters
import pyrogram
from pyrogram.types import ReplyKeyboardMarkup, InlineKeyboardMarkup, InlineKeyboardButton
import csv
import datetime
import asyncio

client_app = Client("my_account", api_id=api_id, api_hash=api_hash)
bot_app = Client("my_bot", api_id=api_id, api_hash=api_hash, bot_token=bot_token)


async def get_message_type(message):
    types = {
        'text': message.text,
        'photo': message.photo,
        'video': message.video,
        'document': message.document,
        'voice': message.voice # ? Need to check
    }

    for key, val in types.items():
        if val: return key


async def convert_chat_type(input_type):
    chat_types = {'work': 'Робочий чат 💼', 'client': 'Чат із клієнтами 💰'}
    return chat_types.get(input_type)

async def get_column_data(database, column_title, condition=None):
    reader = csv.DictReader(open(database, newline='', encoding="utf-8"), delimiter=',')
    try:
        result_arr = list()
        for row in reader:
            if condition in row.values():
                result_arr.append(row.get(column_title))
            elif not condition:
                result_arr.append(row.get(column_title))
        return result_arr
    except:
        return None


async def add_row(database, *args):
    with open(database, 'a', newline = '', encoding="utf-8") as csvfile:
        fieldnames = csv.DictReader(open(database, newline='', encoding="utf-8")).fieldnames
        writer = csv.DictWriter(csvfile, delimiter=",", fieldnames=fieldnames)
        if (database == 'workers.csv'):
            user_data = args[0]
            writer.writerow(
                {
                    'telegram_id': user_data['telegram_id'],
                    'first_name': user_data['first_name'],
                    # 'can_promote': user_data['can_promote'],
                }
            )
        elif (database == 'chats.csv'):
            chat_data = args[0]
            writer.writerow(
                {
                'chat_id': chat_data['chat_id'],
                'chat_title': chat_data['chat_title'],
                'chat_type': chat_data['chat_type'],
                }
            )


async def get_row(database, *args):
    reader = csv.DictReader(open(database, newline='', encoding="utf-8"), delimiter=',')
    if database == 'workers.csv':
        message = args[0]
        for row in reader:
            if str(message) in row['telegram_id']:
                return row
            try:
                if str(message.chat.id) in [row['telegram_id']]:
                    return row
            except AttributeError:
                pass
        return dict()
    if database == 'chats.csv':
        chat = args[0]
        for row in reader:
            if str(chat) == row['chat_id']:
                return row
    return None
    

async def edit_row(database, changed_id, changed_key, changed_value):
    databases = {
        'workers.csv': 'telegram_id',
        'chats.csv': 'chat_id',
    }
    with open(database, newline='', encoding="utf-8") as f:
        data = [r for r in csv.DictReader(f, delimiter=',')]
        for row in data:
            if (row[databases.get(database)] == str(changed_id)):
                row[changed_key] = changed_value; break
    with open(database, 'w', newline = '', encoding="utf-8") as f:
        writer_obj = csv.DictWriter(f, delimiter=',', fieldnames=data[0].keys())
        writer_obj.writeheader()
        writer_obj.writerows(data)


async def remove_row(database, requested_id):
    databases = {
        'workers.csv': 'telegram_id',
        'chats.csv': 'chat_id',
    }
    with open(database, newline='', encoding="utf-8") as f:
        data = [r for r in csv.DictReader(f, delimiter=',')]
        result = list()
        for row in data:
            if (row[databases.get(database)] != str(requested_id)):
                result.append(row)
    with open(database, 'w', newline = '', encoding="utf-8") as f:
        writer_obj = csv.DictWriter(f, delimiter=',', fieldnames=data[0].keys())
        writer_obj.writeheader()
        writer_obj.writerows(result)


@bot_app.on_message(filters.command(['start']))
async def start_worker(bot, message):
    pass


@bot_app.on_message(filters.command(['help']))
async def help_worker(bot, message):
    manual = "1"
    await bot_app.send_message(message.chat.id, text=manual)


@bot_app.on_message(filters.command(['add_chat']))
async def addchat_worker(bot, message):
    input_chat_id = message.text.split(' ')[-1]

    if (not '-' in input_chat_id) and (message.chat.id != message.from_user.id):
        input_chat_id = message.chat.id
    
    # define chat type
    if message.text.split(' ')[-1] in ['work', 'client']:
        chat_type = message.text.split(' ')[-1]
    else:
        chat_type = 'client'
    
    if str(input_chat_id) not in await get_column_data('chats.csv', 'chat_id'):
        await add_row('chats.csv', {'chat_id': input_chat_id, 'chat_title': None, 'chat_type': chat_type})
        msg = f"Додано чат {input_chat_id} в БД бота. Призначення: {await convert_chat_type(chat_type)}"
    else:
        await edit_row('chats.csv', input_chat_id, 'chat_type', chat_type)
        msg = f"Чат {input_chat_id} вже існує в БД бота. Призначення: {await convert_chat_type(chat_type)}"

    await bot_app.send_message(message.chat.id, text=msg, reply_to_message_id=message.id)


@bot_app.on_message(filters.command(['remove_chat']))
async def removechat_worker(bot, message):
    match len(message.text.split(' ')):
        case 2:
            input_chat_id = message.text.split(' ')[-1]
        case 1:
            input_chat_id = message.chat.id

    chat_row = await get_row('chats.csv', input_chat_id)
    try:
        await remove_row('chats.csv', input_chat_id)
        msg = f"Чат {chat_row.get('chat_title')} ({input_chat_id}) було вилучено з бази бота."
        await bot_app.send_message(message.chat.id, text=msg, reply_to_message_id=message.id)
    except Exception as ex:
        msg = f"Помилка: {ex}. Нічого не відбулося."
        await bot_app.send_message(message.chat.id, text=msg, reply_to_message_id=message.id)


@bot_app.on_message(filters.command(['add_worker']))
async def addworker_worker(bot, message):
    worker_id = message.text.split(' ')[1]

    if worker_id == 'me':
        worker_id = message.from_user.id

    if await get_row('workers.csv', worker_id):
        msg = f"⚠️ Увага: користувач із цим ID вже занесений в БД як співробітник. Нічого не відбулося."
    else:
        msg = f"✅ Користувача {worker_id} було додано до переліку співробітників закладу! Його повідомлення не будуть надходити до робочіх чатів."
        await add_row('workers.csv', {'telegram_id': worker_id, 'first_name': None})
    
    await bot_app.send_message(message.chat.id, text=msg, reply_to_message_id=message.id)


@bot_app.on_message(filters.command(['remove_worker']))
async def removeworker_worker(bot, message):
    worker_id = message.text.split(' ')[1]
    if worker_id == 'me':
        worker_id = message.from_user.id
    if await get_row('workers.csv', worker_id):
        msg = f"❌ Користувача {worker_id} було вилучно зі списку співробітників закладу."        
        await remove_row('workers.csv', worker_id)
    else:
        msg = f"⚠️ Увага: не вдалося знайти користувача з цим ID. Нічого не відбулося."
    await bot_app.send_message(message.chat.id, text=msg, reply_to_message_id=message.id)


@bot_app.on_message(filters.command(['chat_list', 'chats']))
async def send_chats(bot, message):
    chats = {'work': list(), 'client': list()}
    reader = csv.DictReader(open('chats.csv', newline='', encoding="utf-8"), delimiter=',')
    for row in reader:
        chats[row['chat_type']].append(row)

    msg = "💬 **Клієнтські чати:**"+ '\n'+'-'*20+'\n'
    for num, chat in enumerate(chats.get('client'), start=1):
        msg += f"{num}) {chat['chat_title']} (`{chat['chat_id']}`)"+'\n'
    await bot_app.send_message(message.chat.id, text=msg)

    msg = "💼 **Робочі чати:**"+ '\n'+'-'*20+'\n'
    for num, chat in enumerate(chats.get('work'), start=1):
        msg += f"{num}) {chat['chat_title']} (`{chat['chat_id']}`)"+'\n'
    await bot_app.send_message(message.chat.id, text=msg)


@bot_app.on_message(filters.command(['worker_list', 'workers']))
async def send_chats(bot, message):
    reader = csv.DictReader(open('workers.csv', newline='', encoding="utf-8"), delimiter=',')
    msg = "👤 **Співробітники room15:**"+ '\n'+'-'*20+'\n'
    for num, row in enumerate(reader, start=1):
        msg += f"{num}) {row['first_name']} (`{row['telegram_id']}`)"+'\n'
    await bot_app.send_message(message.chat.id, text=msg)


@bot_app.on_message(filters.command(['get_chat_id', 'get_id', 'id']))
async def send_group_chat_id(bot, message):
    msg = f"**Current Chat ID:** `{message.chat.id}`"+'\n'
    msg += f"**User's ID:** `{message.from_user.id}`"
    await bot_app.send_message(message.chat.id, text=msg, reply_to_message_id=message.id) 

@bot_app.on_message(filters.group)
async def group_worker(bot, message):
    # if not await get_row('workers.csv', message): # Якщо коментар написав користувач, якого нема у вайтлісті
    #     await bot_app.forward_messages(chat_id=message.chat.id, message_ids=message.id)
    keys = [[ # Second row
                        InlineKeyboardButton(
                            text="🗑 Видалити повідомлення",
                            callback_data=f"delete_message={message.id}"),
                    ]]
    chat_row = await get_row('chats.csv', str(message.chat.id))
    worker_row = await get_row('workers.csv', str(message.from_user.id))
    
    # ** Перевірка на зміну назви бесіди чи імені працівника
    if chat_row.get('chat_title') != message.chat.title:
        await edit_row('chats.csv', message.chat.id, 'chat_title', message.chat.title)
    if worker_row.get('first_name') != message.from_user.first_name:
        await edit_row('workers.csv', message.from_user.id, 'first_name', message.from_user.first_name)

    if chat_row['chat_type'] == 'client' and str(message.from_user.id) not in await get_column_data('workers.csv', column_title='telegram_id'):
        for work_chat_id in await get_column_data('chats.csv', 'chat_id', condition='work'):
            try:
                msg_type = await get_message_type(message) # return msg type like text, photo, video etc
                chat_row = await get_row('chats.csv', str(message.chat.id))
                try: last_name = message.from_user.last_name 
                except: last_name = '' 

                if msg_type == 'text':
                    msg = message.text+'\n\n'
                    msg += f" 💬 `{chat_row.get('chat_title')}` | {message.from_user.first_name} {str(last_name).replace('None', '')}"
                    await bot_app.send_message(int(work_chat_id), text=msg, reply_markup=InlineKeyboardMarkup(keys))
                elif msg_type in ['photo', 'video', 'document']:
                    msg = message.caption+'\n\n'
                    msg += f" 💬 `{chat_row.get('chat_title')}` | {message.from_user.first_name} {str(last_name).replace('None', '')}"
                    if msg_type == 'photo':
                        await bot_app.send_photo(int(work_chat_id), caption=msg, photo=message.photo.file_id, reply_markup=InlineKeyboardMarkup(keys))
                    elif msg_type == 'video':
                        await bot_app.send_video(int(work_chat_id), caption=msg, video=message.video.file_id, reply_markup=InlineKeyboardMarkup(keys))
                    elif msg_type == 'document':
                        await bot_app.send_document(int(work_chat_id), caption=msg, document=message.document.file_id, reply_markup=InlineKeyboardMarkup(keys))
                elif msg_type == 'voice':
                    msg = f" 💬 `{chat_row.get('chat_title')}` | {message.from_user.first_name} {str(last_name).replace('None', '')}"
                    await bot_app.send_voice(int(work_chat_id), caption=msg, voice=message.voice.file_id, reply_markup=InlineKeyboardMarkup(keys))
                
                
                
                # await bot_app.forward_messages(int(work_chat_id), from_chat_id=message.chat.id, message_ids=message.id)
            except:
                pass

    # reader = csv.DictReader(open('chats.csv', newline='', encoding="utf-8"), delimiter=',')
    # for row in reader:
    #     if not row.get('chat_title'):

@bot_app.on_callback_query()
async def callbcak_worker(bot, call):
    if 'delete_message' in call.data:
        await bot_app.delete_messages(chat_id=call.message.chat.id, message_ids=call.message.id)


bot_app.start()
pyrogram.idle()
bot_app.stop()